'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _restify = require('restify');

var _restify2 = _interopRequireDefault(_restify);

var _mongoose = require('mongoose');

var _mongoose2 = _interopRequireDefault(_mongoose);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

var _socket3 = require('socket.io-client');

var _socket4 = _interopRequireDefault(_socket3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Mighty = function () {
  function Mighty(config) {
    _classCallCheck(this, Mighty);

    var base_dir = __dirname;
    this.argv = (0, _minimist2.default)(process.argv);
    var _this = this;
    this.baseConfig = require('./config.js').default;
    if (config) {
      this.config = config;
    } else {
      this.config = this.baseConfig;
    }

    if (!this.argv.e) {
      this.argv.e = 'development';
    }
    if (!this.argv.p) {
      this.argv.p = process.cwd();
    }

    this.app = null;
    this.environment = this.argv.e;
    this.cwd = this.argv.p;
    this.modules = {};
    this.models = {};
    _mongoose2.default.Promise = global.Promise;
    this.db = _mongoose2.default.connect(this.config.database.url + this.config.data_space);
    console.log('Trying to connect to DB: ' + this.config.database.url + this.config.data_space);

    _restify2.default.CORS.ALLOW_HEADERS.push('accept');
    _restify2.default.CORS.ALLOW_HEADERS.push('sid');
    _restify2.default.CORS.ALLOW_HEADERS.push('lang');
    _restify2.default.CORS.ALLOW_HEADERS.push('origin');
    _restify2.default.CORS.ALLOW_HEADERS.push('withcredentials');
    _restify2.default.CORS.ALLOW_HEADERS.push('x-requested-with');
    _restify2.default.CORS.ALLOW_HEADERS.push('apikey');
    _restify2.default.CORS.ALLOW_HEADERS.push('access-token');
    _restify2.default.CORS.ALLOW_HEADERS.push('authentication-key');
    _restify2.default.CORS.ALLOW_HEADERS.push('session-id');
    _restify2.default.CORS.ALLOW_HEADERS.push('authentication');

    if (this.config.CORS_HEADERS) {
      for (var i = 0; i < this.config.CORS_HEADERS.length; i++) {
        _restify2.default.CORS.ALLOW_HEADERS.push(this.config.CORS_HEADERS[i]);
      }
    }

    this.app = _restify2.default.createServer({ name: this.config.application_name, version: _package2.default.version });
    this.app.use(_restify2.default.CORS({
      origins: this.config.CORS_ORIGINS ? this.config.CORS_ORIGINS : ['*'],
      credentials: true
    }));
    this.app.use(_restify2.default.fullResponse());
    this.app.use(_restify2.default.acceptParser(this.app.acceptable));
    this.app.use(_restify2.default.bodyParser({ overrideParams: true }));
    this.app.use(_restify2.default.queryParser());
    this.app.use(_restify2.default.gzipResponse());

    this.core = {
      api: require(base_dir + '/api').API,
      registrar: require(base_dir + '/registrar').Registrar,
      utils: require(base_dir + '/utils')
    };
    this.app.use(this.core.utils.default.beforeConditions);

    this.db.plugin(require('./db_utils'));
    // Setup some basic methods

    this.io = _socket2.default.listen(3459);
    this.io.use(require('socketio-wildcard')());
    return new Promise(function (resolve, reject) {
      return _this.start().then(function (mighty) {
        var User = mighty.db.model('User');

        User.find().limit(1).then(function (users) {
          if (!users.length) {
            console.log('Waiting for first user');
            mighty.first_user = true;
          }
          return resolve(mighty);
        }).catch(function (err) {
          console.log(err);
          return reject(err);
          process.exit(1);
        });
      });
    });
  }

  _createClass(Mighty, [{
    key: 'loadModules',
    value: function loadModules(folder) {
      var _this = this;
      var modules = _fs2.default.readdirSync(folder);
      modules.forEach(function (filename) {
        var filepath = folder + '/' + filename;
        if (_fs2.default.statSync(filepath).isDirectory()) {
          try {
            _fs2.default.accessSync(filepath + '/index.js', _fs2.default.F_OK);
            var m = require(filepath);
            _this.modules[filename] = new m(_this);
            if (_this.modules[filename].config) {
              _this.modules[filename].api = new _this.core.api(_this, _this.modules[filename].config).loadAPI();
              _this.models[_this.modules[filename].config.data_space] = _this.modules[filename].api.model;
            }
          } catch (err) {
            return console.log('Module: ' + filepath + ' had a problem:\
          ' + err.message + '\
          ' + err.stack + '\
          ');
          }
        }
      });
    }
  }, {
    key: 'start',
    value: function start() {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var _this = _this2;

        _this2.db.connection.once('open', function () {
          // we want to load our core modules first.
          if (_this.config.connection.type === 'master') {
            _this.loadModules(_this.baseConfig.module_folder);
          } else if (_this.config.connection.type === 'slave') {
            (function () {
              var master_host = _this.config.connection.master_host;
              if (!master_host) {
                if (_this.environment === 'production') {
                  master_host = 'https://app.mightyspace.xyz'; // production container
                } else if (_this.environment === 'testing') {
                  master_host = 'https://testing.mightyspace.xyz'; // testing container
                } else {
                  master_host = 'http://mighty-core:3459'; // developer container
                }
              }
              // connect to the "master_host"
              _this.master_socket = (0, _socket4.default)(master_host);
              console.log('Attempting to connect to Master Host: ' + master_host);
              _this.master_socket.on('error', function (err) {
                console.log(err);
              });
              _this.master_socket.on('connect', function () {
                console.log('Connected to Master Host: ' + master_host);
              });
            })();
          }

          _this.app.listen(_this.config.api_port, function () {
            console.log('%s listening at %s in %s mode', _this.config.application_name, 'http://' + _this.config.application_host + ':' + _this.config.api_port, _this.environment);
            return resolve(_this);
          });
        });
      });
    }
  }, {
    key: 'getModel',
    value: function getModel(data_space) {
      return this.models[data_space];
    }
  }]);

  return Mighty;
}();

exports.default = Mighty;