'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
  * Authentication methods and middleware
  *
  * @class AuthModule
  *
  */
var AuthModule = function () {
  function AuthModule(mighty) {
    _classCallCheck(this, AuthModule);

    this.app = mighty;
    var _this = this;

    mighty.app.use(function (req, res, next) {
      if (!mighty.first_user) {
        return next();
      }
      if (req.url.indexOf('/auth/register') > -1) {
        req.params.role = 'admin';
        return next();
      }
      res.json({ register_first_user: true });
    });

    mighty.app.get('/v', function (req, res, next) {
      res.json({ name: mighty.app.name, version: mighty.config.version });
    });

    mighty.app.use(function (req, res, next) {
      if (!req.route.config || !req.route.config.authentication) {
        return next();
      }
      // check for valid user
      if (!req.headers['authentication-key']) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Missing Authentication Key header' });
      }

      var User = _this.app.db.model('User');
      User.findOne({ authentication_key: req.headers['authentication-key'] }).then(function (user) {
        if (!user) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid Authentication Key' });
        }

        req.user = user;
        return next();
      }).catch(function (err) {
        return _this.app.core.api.handleError(res, { code: 500, message: err.message });
      });
    });
    mighty.app.use(function (req, res, next) {
      if (req.route.config.admin_prefix && req.route.path.indexOf(req.route.config.admin_prefix) <= -1) {
        return next();
      }
      if (!req.route.config || !req.route.config.admin) {
        return next();
      }

      if (!req.user) {
        return _this.app.core.api.handleError(res, { code: 403, message: 'Invalid access' });
      }
      if (req.user.role !== 'admin') {
        return _this.app.core.api.handleError(res, { code: 403, message: 'Invalid access' });
      } else {
        return next();
      }
    });
    mighty.app.use(function (req, res, next) {
      if (!req.route.config || !req.route.config.owner) {
        return next();
      }
      if (!req.user) {
        return _this.app.core.api.handleError(res, { code: 403, message: 'Forbidden' });
      }
      switch (req.route.method) {
        case 'GET':
          req.params.conditions = { _owners: req.user._id };
          break;
        case 'POST':
          req.params._owners = [req.user._id];
          break;
        case 'PUT':
          req.params.conditions = { _owners: req.user._id };
          break;
        case 'DELETE':
          req.params.conditions = { _owners: req.user._id };
          break;
      }

      return next();
    });
    mighty.app.use(function (req, res, next) {
      if (!req.route.config || !req.route.config.organization) {
        return next();
      }

      // check for valid organization and permission
      if (!req.headers['organization-id']) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Missing organization ID' });
      }
      if (!req.user) {
        return _this.app.core.api.handleError(res, { code: 403, message: 'Missing user authentication' });
      }
      var orgId = void 0;
      try {
        orgId = _this.app.db.Types.ObjectId(req.headers['organization-id']);
        if (req.user.organizations.indexOf(orgId) < 0) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'User is not in this organization' });
        }
      } catch (err) {
        return _this.app.core.api.handleError(res, { code: 500, message: 'Malformed organization ID or problem with User document' });
      }

      var Organization = _this.app.db.model('Organization');
      Organization.findOne({ _id: orgId }).then(function (org) {
        if (!org) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid organzation ID' });
        }

        req.organization = org;
        return next();
      }).catch(function (err) {
        return _this.app.core.api.handleError(res, { code: 500, message: err.message });
      });
    });
  }

  _createClass(AuthModule, null, [{
    key: 'generateAuthenticationKey',
    value: function generateAuthenticationKey(email, secret) {
      var time = new Date().getTime();
      return _crypto2.default.createHash('md5').update(email + time + secret).digest("hex");
    }
  }, {
    key: 'generatePassword',
    value: function generatePassword(secret, password_text) {
      return new Promise(function (resolve, reject) {
        _crypto2.default.pbkdf2(password_text, secret, 10000, 512, 'sha512', function (err, encodedPassword) {
          if (err) {
            return reject(err);
          }
          return resolve(new Buffer(encodedPassword, 'base64').toString('hex'));
        });
      });
    }
  }]);

  return AuthModule;
}();

module.exports = AuthModule;