'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var Schema = require('mongoose').Schema;

var config = {
  module_name: 'Organizations',
  data_space: 'organizations',
  route_space: '/organizations',
  version: '1.0.0',
  dependencies: {
    // 'instrmnt.other_module': 'x.x.x' -- use the data_space
  },
  model: {
    // Mongoose model
    name: 'Organization',
    schema: {
      _owners: [{ type: Schema.Types.ObjectId, ref: 'User' }],
      name: { type: String, required: true },
      billing_address: {
        company_name: String,
        attention_to: String,
        email: String,
        phone: String,
        address: {
          address_1: String,
          address_2: String,
          address_3: String,
          city: String,
          state_code: String,
          postal_code: String,
          country_code: String
        },
        residential: false,
        lift_gate: false,
        call_before: false,
        inside_delivery: false
      },
      shipping_address: {
        company_name: String,
        attention_to: String,
        email: String,
        phone: String,
        address: {
          address_1: String,
          address_2: String,
          address_3: String,
          city: String,
          state_code: String,
          postal_code: String,
          country_code: String
        },
        residential: false,
        lift_gate: false,
        call_before: false,
        inside_delivery: false
      },
      phone_number: String,
      website: String
    }
  },
  api: {
    get: {
      enabled: true,
      authentication: true,
      owner: true,
      admin: true
      // jsapi: false,
      // jsapi_authentication: false,
      // socket: false,
      // beforeConditions: {},
      // afterConditions: {}
    },
    post: {
      enabled: true,
      authentication: true,
      owner: true,
      afterConditions: {
        $push: { request: 'user', field: 'organizations', value: '_id', save: true } // this is some crazy syntax we need to document
      },
      admin: true
    },
    put: {
      enabled: true,
      authentication: true,
      owner: true,
      admin: true
    },
    del: {
      enabled: true,
      authentication: true,
      owner: true,
      admin: true
    }
  }
};

exports.default = config;