'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _auth = require('../auth');

var _auth2 = _interopRequireDefault(_auth);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var RegistryRegisterModule = function RegistryRegisterModule(app) {
  var _this2 = this;

  _classCallCheck(this, RegistryRegisterModule);

  this.app = app;
  this.registry = {};
  var _this = this;

  this.createRegistration = function (params, options) {
    return new Promise(function (resolve, reject) {
      return resolve();
    });
  };

  this.handleRegistration = function (req, res, next) {
    if (!_validator2.default.isEmail(req.params.app_name + '')) {
      return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid app name' });
    }

    if (!_validator2.default.isURL(req.params.app_repository + ''), { protocols: ['https'] }) {
      return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid Application Repository: should be a fully valid URL (example: https://my_git/repository.git)' });
    }

    _this.createRegistration(req.params).then(function (registration) {
      return res.json(registration);
    }).catch(function (err) {
      _this.app.core.api.handleError(res, { code: 500, message: err.message });
    });
  };

  /**
    * Register a service to be used in the ecosystem.
    *
    * @param config {Object}
    * @param config.namespace {String}
    * @param config.events {Object} Key pair `{'event-name': options}` where options is an object passed to the service along with the data. TODO: This can be used to grab other information before sending the service call.
    *
    */
  this.registerService = function (config, socket) {
    return new Promise(function (resolve, reject) {
      if (!config.name) {
        return reject({ code: 404, message: 'Missing service name' });
      }
      if (!(config.events instanceof Array)) {
        return reject({ code: 404, message: 'Missing service events' });
      }
      /* TODO: Add authorization to services, until then allow services to be overridden
      if(_this.registry[config.name]) {
        return reject({code: 404, message: `Service is already registered, please use the 'service.update' event if you need to make changes`});
      }
      */

      _this.registry[config.name] = {
        events: config.events,
        socket: socket
      };
      return resolve(_this.registry[config.name]);
    });
  };

  this.handleServiceRegistration = function (data, socket) {
    return new Promise(function (resolve, reject) {
      _this.registerService(data, socket).then(function (result) {
        return resolve({ ok: true });
      }).catch(function (err) {
        return reject(err);
      });
    });
  };

  this.handleServiceCall = function (serviceSocket, evt, data, socket) {
    if (!serviceSocket) {
      return socket.emit('service.error', { code: 500, message: 'Missing or disconnected service socket' });
    }
    // For when the service is `eventName.done` this should be called.
    serviceSocket.once(evt + '.done', function (req) {
      console.log(req);
      socket.emit(evt, req);
    });
    serviceSocket.emit(evt, data);
  };

  app.io.on('connection', function (socket) {
    socket.on('*', function (req) {
      var eventName = void 0;
      var data = void 0;

      try {
        eventName = req.data[0];
        data = null;
        if (req.data[1] && !(req.data[1] instanceof Object)) {
          data = JSON.parse(req.data[1]);
        } else {
          data = req.data[1];
        }
      } catch (err) {
        console.log(err);
        return socket.emit('service.error', { code: 500, message: 'Invalid event name or bad JSON packet' });
      }

      if (eventName.indexOf('.done') > -1) {
        // completed events don't need to be handled here.
        return;
      }

      switch (eventName) {
        case 'service.register':
          _this2.handleServiceRegistration(data, socket).then(function (result) {
            socket.emit('service.register.' + data.name, result);
          }).catch(function (err) {
            socket.emit('service.error', err);
          });
          break;
        default:
          var eventParts = eventName.split('.');
          if (_this.registry[eventParts[0]] && _this.registry[eventParts[0]].events.indexOf(eventName) > -1) {
            return _this.handleServiceCall(_this.registry[eventParts[0]].socket, eventName, data, socket);
          }
          socket.emit('service.error', { code: 404, message: 'Service call `' + eventName + '` unknown' });
          break;
      }
    });
  });
};

exports.default = RegistryRegisterModule;
;